import { Inject, Injectable } from "@nestjs/common";
import { AddedJudgementDto } from './dto/added-judgement.dto';
import { UpdateJudgementDto } from './dto/update-judgement.dto';
import {
  AddedNewJudgementResponseType,
  GetListJudgementsResponseType,
  GetOneJudgementResponseType, GetPaginatedListOfJudgementsResponse,
  RemoveJudgementResponse, UpdateJudgementResponse
} from "../types";
import { Judgement } from "./entities/judgement.entity";
import { User } from "../user/entities/user.entity";
import { UserService } from "../user/user.service";

@Injectable()
export class JudgementsService {
  constructor(
    @Inject(UserService) private userService: UserService,
  ) {}

  async addJudgement(newJudgement: AddedJudgementDto, user: User): Promise<AddedNewJudgementResponseType> {
    const {
      currency,
      yearAgreement,
      bank,
      stateOfTheMatter,
      creditType,
      adjudication,
      signature,
      judge,
      court,
      linkToTheJudgment
    } = newJudgement;

    if (
      typeof currency !== 'string'
      ||
      typeof creditType   !== 'string'
      ||
      typeof bank  !== 'string'
      ||
      typeof stateOfTheMatter  !== 'string'
      ||
      typeof adjudication  !== 'string'
      ||
      typeof signature  !== 'string'
      ||
      typeof judge  !== 'string'
      ||
      typeof court  !== 'string'
      ||
      typeof linkToTheJudgment !== 'string'
      ||
      typeof yearAgreement !== 'number'
      ||
      currency !== 'CHF'
      &&
      currency !== 'USD'
      &&
      currency !== 'JPY'
      &&
      currency !== 'EUR'
      ||
      creditType !== 'indeksowany'
      &&
      creditType !== 'denominowany'
      ||
      judge === ''
      ||
      court === ''
      ||
      signature === ''
      ||
      yearAgreement < 2000
      ||
      yearAgreement > 2012
    ) {
      return {
        successfullyCompleted: false,
      };
    }

    const newVerdict = new Judgement();
    newVerdict.currency = currency;
    newVerdict.yearAgreement = yearAgreement;
    newVerdict.bank = bank;
    newVerdict.stateOfTheMatter = stateOfTheMatter;
    newVerdict.creditType = creditType;
    newVerdict.month = new Date().getMonth();
    newVerdict.year = new Date().getFullYear();
    newVerdict.adjudication = adjudication;
    newVerdict.signature = signature;
    newVerdict.judge = judge;
    newVerdict.court = court;
    newVerdict.linkToTheJudgment = linkToTheJudgment;
    newVerdict.createdBy = user.name;

    await newVerdict.save();

    return {
      successfullyCompleted: true,
      id: newVerdict.id,
      bank: newVerdict.bank,
      adjudication: newVerdict.adjudication,
      signature: newVerdict.signature,
      judge: newVerdict.judge,
      court: newVerdict.court
    };
  }

  async showAllJudgements(): Promise<GetListJudgementsResponseType> {
    return await Judgement.find();
  }

  async showNineJudgementsPerPage(currentPage: number = 1): Promise<GetPaginatedListOfJudgementsResponse> {
    const maxPerPage = 9;

    const [judgements, count] = await Judgement.findAndCount({
      skip: maxPerPage * (currentPage - 1),
      take: maxPerPage,
    });

    const pagesCount = Math.ceil(count/maxPerPage);

    return {
      judgements,
      pagesCount
    };
  }

  async showOneJudgements(id: string): Promise<GetOneJudgementResponseType> {
    return await Judgement.findOne(id);
  }

  async updateOneJudgement(id: string, updateJudgement: UpdateJudgementDto, user): Promise<UpdateJudgementResponse> {
    const {currency, yearAgreement, bank, stateOfTheMatter, creditType, month, year, adjudication, signature, judge, court, linkToTheJudgment} = updateJudgement;

    if (
      typeof currency !== 'string'
      ||
      typeof creditType   !== 'string'
      ||
      typeof bank  !== 'string'
      ||
      typeof stateOfTheMatter  !== 'string'
      ||
      typeof adjudication  !== 'string'
      ||
      typeof signature  !== 'string'
      ||
      typeof judge  !== 'string'
      ||
      typeof court  !== 'string'
      ||
      typeof linkToTheJudgment !== 'string'
      ||
      typeof yearAgreement !== 'number'
      ||
      typeof month  !== 'number'
      ||
      typeof year  !== 'number'
      ||
      currency !== 'CHF'
      &&
      currency !== 'USD'
      &&
      currency !== 'JPY'
      &&
      currency !== 'EUR'
      ||
      creditType !== 'indeksowany'
      &&
      creditType !== 'denominowany'
      ||
      judge === ''
      ||
      court === ''
      ||
      signature === ''
      ||
      yearAgreement < 2000
      ||
      yearAgreement > 2012
    ) {
      return {
        successfullyCompleted: false,
      };
    }

    await Judgement.update(id, {
      currency,
      yearAgreement,
      bank,
      stateOfTheMatter,
      creditType,
      month,
      year,
      adjudication,
      signature,
      judge,
      court,
      linkToTheJudgment,
      updateBy: user.name
    });

    return {
      successfullyCompleted: true,
      id,
      bank: updateJudgement.bank,
      adjudication: updateJudgement.adjudication,
      signature: updateJudgement.signature,
      judge: updateJudgement.judge,
      court: updateJudgement.court
    };
  }

  async removeOneJudgement(id: string): Promise<RemoveJudgementResponse> {
    const removeJudgement = await Judgement.findOne({
      where: {
        id
      },
    });

    if (removeJudgement) {
      await removeJudgement.remove();

      return {
        successfullyCompleted: true,
        id
      };
    } else {
      return {
        successfullyCompleted: false
      }
    }
  }
}
